Program HMO ;
{ Calculation of HMO  by  H. KIHARA }
{ Console application by Free Pascal }

type   Matrix = array[1..30,1..30] of real ;

var    compound : string[40] ;    { compound name }
       h        : Matrix ;        { secular determinat }
       v        : Matrix ;        { coefficient matrix }
       p        : Matrix ;        { bondorder matrix }
       natom    : integer ;       { number of atoms }
       nbond    : integer ;       { number of bonds }
       npi_elec : integer ;       { number of electrons }
       nhetero  : integer ;       { number of matrix elements to be changed }
       i,j,k  : integer ;
       hij      : real ;
       q        : char;

procedure  Jacobi(var h,v:Matrix ; n:integer) ;

{ Diagonalization of matrix by Jacobi's method }

const threshold = 0.00001 ;
      itmax = 100 ;               { maximaum iteration cycles }

var   d,sm,hj,hk,ta,tb,sn,cs,hkl,hjl,vj,vk : real ;
      i,j,l,it : integer ;

begin
   it := 1 ;
   d := 1.0 ;
   for i := 1 to n do
      for j := 1 to n do
         v[i,j] := 0.0 ;
   for i := 1 to n do v[i,i] := 1.0 ;        { Identity matrix }
   while (d>threshold) and (it<itmax) do     { Convergence judgment }
   begin
      d := abs(h[2,1]) ;
      for i := 2 to n do
      begin
         for j := 1 to i-1 do  if abs(h[i,j])>d then d := abs(h[i,j])
      end ; { of for i }
      if d > threshold then
      begin
         sm := 0.1*d ;
         it := it+1 ;
         for j := 2 to n do
         begin
            for k := 1 to j-1 do
            begin
               if abs(h[j,k]) > sm then
               begin
                  hj := h[j,j] ;
                  hk := h[k,k] ;
                  tb := (hj-hk)/(2.0*h[k,j]) ;
                  if tb<=0 then ta := tb-sqrt(1.0+tb*tb)
                           else ta := tb+sqrt(1.0+tb*tb) ;
                  cs := 1.0/sqrt(1.0+ta*ta) ;
                  sn := cs*ta ;
                  for l := 1 to n do
                  begin
                     if l=k then
                     begin
                        h[k,k] := hk*cs*cs+h[j,k]*2.0*sn*cs+hj*sn*sn ;
                        h[j,j] := hk+hj-h[k,k] ;
                     end  { of if }
                     else if l<>j then
                     begin
                        hkl := h[k,l] ;
                        hjl := h[j,l] ;
                        h[k,l] := hkl*cs+hjl*sn ;
                        h[j,l] := hjl*cs-hkl*sn ;
                        h[l,k] := h[k,l] ;
                        h[l,j] := h[j,l]
                     end ; { of if }
                  end ; { of for L }
                  h[j,k] := h[j,k]*(cs*cs-sn*sn)+(hj-hk)*sn*cs ;
                  h[k,j] := h[j,k] ;
                  for i := 1 to n do
                  begin
                     vj := v[i,j] ;
                     vk := v[i,k] ;
                     v[i,j] := vj*cs-vk*sn ;
                     v[i,k] := vj*sn+vk*cs ;
                  end ; { of for i }
               end ; { of if }
            end ; {of for  k }
         end ; { of for j }
      end ; { of if }
   end ; { of while }
end ; { of Jacobi }

procedure swap_val(var a,b : real) ;

var temp : real ;

begin
   temp := a ;
   a := b ;
   b := temp ;
end ; { of SWAP_VAL }

procedure sort(var h,v : Matrix ; n :integer) ;

{ Sort eigenvalues and eigenvectors in ascending order of eigenvalues }

var   i,j,max : integer ;
      emax : real ;

begin
   for i := 1 to n do
   begin
      emax := h[i,i] ;
      max := i ;
      for j := i to n do
      begin
         if h[j,j] > emax then
         begin
            max := j ;
            emax := h[j,j] ;
         end ; { of if }
      end ; { of for }
      if i< max then
      begin
         swap_val(h[i,i],h[max,max]) ;
         for j := 1 to n do  swap_val(v[j,i],v[j,max]) ;
      end ; { of if }
   end ; { of for }
end ; { of SORT }

procedure Print_Matrix(var h,v : Matrix ; n : integer) ;

{ Print eigenvalues and eigenvectors }

var  first,last,column : integer ;
     i,j,k : integer ;

begin
   column := 8 ;          { number of items in a line}
   first := 1 ;
   last := first+column - 1 ;
   writeln ;
   writeln('--  Eigen values and Eigen vector   --') ; writeln ;
   while first<=n do
   begin
      writeln ;
      if last>n then last := n ;
      write('    ');
      for i := first to last  do write(h[i,i]:8:4) ;
      writeln ;
      for k := 1 to n do
      begin
         write(k:4) ;
         for j := first to last do write(v[k,j]:8:4) ;
         writeln ;
      end ; { of for k }
      writeln ;
      first := last + 1 ;
      last := first + column - 1 ;
   end ; { of while }
end ; { of MATPRINT }

procedure pmat(var h,v,p: matrix; n,np: integer) ;

   { Calculation of Bondorder }
    var i, j, k, m, r: integer;
		nocc: integer;
        odd, degenerate: boolean;
        eps: real;
        nitem : integer;

	begin
	   nitem  := 8;
       eps := 0.000001;
       odd := false;
       degenerate := false;

       nocc := (np + 1) div 2;
       if nocc * 2 > np then odd := true;

    { Compare HOMO and LUMO MO energy}
       if (h[nocc, nocc] - h[nocc + 1, nocc + 1]) < eps then degenerate := true;

       for i := 1 to n do
          for j := 1 to n do p[i, j] := 0.0;

       for i := 1 to n do
	     begin
           for j := 1 to i do
	         begin
               for k := 1 to nocc - 1 do  p[i, j] := p[i, j] + 2.0 * v[i, k] * v[j, k];

               if degenerate then
		          begin
                     if odd then p[i, j] := p[i, j] + 0.5 * (v[i, nocc] * v[j, nocc] + v[i, nocc + 1] * v[j, nocc + 1])
                       else p[i, j] := p[i, j] + v[i, nocc] * v[j, nocc] + v[i, nocc + 1] * v[j, nocc + 1]
                  end
               else
	              if odd then p[i, j] := p[i, j] + v[i, nocc] * v[j, nocc]
                     else p[i, j] := p[i, j] + 2.0 * v[i, nocc] * v[j, nocc]

             end ;  { for j}

         end ;   { for i}

     writeln('-- Bond order matrix --');
	writeln ;
    r := 1;
    m := r + nitem - 1;

    while r <= n do
	  begin

        if m > n then m := n;

        write('  ');
        for i:= r to m do write(i:8);
		writeln ; writeln ;

        for i:= 1 to n do
			begin
               write(i:4);
               for j:= r to m do write( p[i, j]:8:4);
 			   writeln
            end;

        r := m + 1;
        m := r + nitem - 1;
 		writeln ;

      end;

 end ;  { of Pmat}

begin
   writeln('  HMO Calculation Program') ;
   writeln ;
   write('Name of Compound  : ') ;
   readln(compound) ;
   write('Number of atoms: ');
   readln(natom) ;
   write('Number of bonds: ');
   readln(nbond) ;
   write('Number of electrons: ');
   readln(npi_elec) ;
   write('Number of matrix elements to be changed: ');
   readln(nhetero) ;

   for i := 1 to natom do
      for j := 1 to natom do
         h[i,j] := 0.0 ;

   writeln('Enter the number pair (i, j) of the atoms bonded.') ;
   for k := 1 to nbond do
   begin
      write('  Bond (',k:2,') :  ') ;
      readln(i,j) ;
      h[i,j] := 1.0 ;
      h[j,i] := 1.0 ;
   end ; { of for }

   if nhetero>0 then
   begin
      writeln('Enter the matrix element to change (i, j, Hij).');
      for k := 1 to nhetero do
      begin
         write(k:4,' : ') ;
         readln(i,j,hij) ;
         h[i,j] := hij ;
         h[j,i] := hij ;
      end ; { of for k }
   end ; { of if }

   Jacobi(h,v,natom) ;

   Sort(h,v,natom) ;

   Print_Matrix(h,v,natom) ;

   pmat(h,v,p,natom,npi_elec) ;

   writeln; write('Hit Return key.  ');
   read(q) ;

end.

